%Reglas de poltica monetaria para una economa abierta
%Autor: Augusto Aliaga
%Fecha: Mayo de 2020
@#define REGLA6 =0
@#define REGLA7 =0
@#define REGLA8 =0
@#define REGLA9 =1
@#define REGLA10 =0

% Cadenas de cada regla de poltica monetaria
@#if REGLA6 == 1
    case_title='REGLA6'; 
    short_title='R6';   
@#else
    @#if REGLA7 ==1
         case_title='REGLA7';
         short_title='R7';
    @#else
        @#if REGLA8 ==1
            case_title='REGLA8';
            short_title='R8';   
        @#else
            @#if REGLA9 ==1
                case_title='REGLA9';
                short_title='R9';
            @#else
                @#if REGLA10 ==1
                     case_title='REGLA10';
                     short_title='R10';
                @#else
                    error
                @#endif
            @#endif
        @#endif       
    @#endif
@#endif
var x                                                               %Brecha del producto
    piD                                                             %Inflacin domstica
    i                                                               %Tasa de inters nominal
    il                                                              %Tasa de interes a largo plazo
    bl                                                              %Bonos a largo plazo
    yN                                                              %Producto natural
    rN                                                              %Tasa de inters natural
    sN                                                              %Terminos de intercambio natural
    sB                                                              %Brecha de trminos de intercambio
    y                                                               %Producto
    ti                                                              %Trminos de intercambio
    pi                                                              %CPI inflacin
    n                                                               %Empleo
    rR                                                              %Tasa de inters real   
    w                                                               %Salario nominal  
    nx                                                              %Exportaciones netas
    c                                                               %Consumo   
    yS                                                              %Producto mundial
    yH                                                              %Desviacin de producto desde su estado estacionario
    pD                                                              %ndice de precios domstico
    p                                                               %CPI
    er                                                              %Tipo de cambio nominal
    Der                                                             %Desviacin del tipo de cambio nominal
    M                                                               %Proceso AR(1) del choque de politica monetaria
    a                                                               %Proceso AR(1) del choque tecnolgico
    g                                                               %Proceso AR(1) del choque de preferencias
    iS                                                              %Proceso AR(1) del choque de tasa de inters internacional
    pS                                                              %Nivel de precios mundial
    ;

varexo eM                                                           %Choque de politica monetaria
       eA                                                           %Choque tecnolgico
       eG                                                           %Choque de preferencias
       eIS                                                          %Choque de tasa de inters internacional
       ePS                                                          %Choque del nivel de precios mundial
       ;

parameters beta                                                     %Factor de descuento
    sigma                                                           %inversa de la EIS
    fi                                                              %inversa de la elasticidad de Frisch
    alfaM                                                           %Proporcin del capital
    epsilon                                                         %Elasticidad de la demanda
    theta                                                           %Parmetro de Calvo 
    alfa                                                            %Parmetro de apertura
    eta                                                             %substitutability foreign/domestic goods
    rhoA                                                            %Parmetro del choque tecnolgico
    rhoM                                                            %Parmetro del choque de poltica monetaria
    rhoG                                                            %Parmetro del choque de preferencias
    rhoIS                                                           %Parmetro del choque de tasa de inters internacional
    rhoPS                                                           %Parmetro del choque del nivel de precios internacional
    phiP                                                            %Reaccin a la inflacin
    phiX                                                            %Reaccin a la brecha del producto
    phiR                                                            %Reaccin al rezago de la tasa de inters nominal
    phiPF                                                           %Reaccin a la inflacin domstica proyectada
    phiER                                                           %Reaccin al tipo de cambio en la regla monetaria
    phiR1
    phiP0
    phiP1
    phiX0
    phiX1
    phiDER
    phiTI
    delta                                                           %Tasa de los costos de ajuste
    ;

%----------------------------------------------------------------
% Parametrizacin
%----------------------------------------------------------------
beta    = 0.9852;
sigma   = 1;
fi      = 0.45;
alfaM   = 0;
epsilon = 6;
theta   = 0.75;
alfa    = 0.6;
rhoM    = 0.5;
rhoA    = 0.9;
rhoG    = 0.9;
rhoIS   = 0.9;
rhoPS   = 0.95;
phiP    = 1.5;
phiX    = 0.5;
phiR    = 0.5;
phiPF   = 5;
phiER   = 0.5;
phiR1   = 0.763;
phiP0   = 0.107;
phiP1   = 0.028;
phiX0   = 0.346;
phiX1   = 0.062;
phiDER  = 0.053;
phiTI   = 0.082;
eta     = 2;
delta   = 3;

%----------------------------------------------------------------
% Modelo log-lineal
%----------------------------------------------------------------
model(linear); 
%Parmetros compuestos
#omicron   = (1-alfaM)/(1-alfaM+alfaM*epsilon);           
#lambda    = (1-theta)*(1-beta*theta)/theta*omicron;     
#omega     = sigma*eta + (1-alfa)*(sigma*eta-1);         
#fim       = 1/(1+alfa*(omega-1));                    
#sigmaA    = sigma*fim;                                  
#gammaA    = (1+fi)/(sigmaA*(1-alfaM)+fi+alfaM);                                            
#gammaS    = - alfa*(omega-1)*sigmaA*(1-alfaM)/(sigmaA*(1-alfaM)+fi+alfaM);           
#gammaG    = - alfa*omega*fim*(1 - alfaM)/(sigmaA*(1-alfaM)+fi+alfaM);                       
#kappaA    = lambda*(sigmaA+(fi+alfaM)/(1 - alfaM));                                           
#psiS      = sigmaA*(alfa*(omega-1)+gammaS);                                                
#psiG      = (1-alfa)*fim - sigmaA*gammaG;
#gamma1   =   (1+delta+beta);                                       
#gamma2    = ((1+delta)^2)/delta;                                                                             
x     = x(+1) - 1/sigmaA*((1/(1+delta))*i - piD(+1) + (delta/(1+delta))*il(+1) - rN);           %Curva IS dinmica
piD   = beta*piD(+1) + kappaA*x;                                                                %Curva de Phillips Mevo Keynesiana                                                                                    
il    = i(-1) + gamma1*gamma2*bl(-1) - gamma2*bl(-2) - beta*gamma2*bl;                          %Tasa de interes a largo plazo
i     = iS + er(+1) - er;                                                                       %Paridad descubierta de tasas de interes
yN    = gammaA*a + gammaG*g + gammaS*yS;                                                        %Producto natural
rN    = -sigmaA*gammaA*(1-rhoA)*a + psiS*(yS(+1)-yS) + psiG*(1-rhoG)*g;                         %Tasa de inters natural
sN    = sigmaA*(yN-yS) - (1-alfa)*fim*g;                                                        %Terminos de intercambio natural                                             
sB    = sigmaA*x;                                                                               %Brecha de trminos de intercambio
x     = y - yN;                                                                                 %Producto
sB    = ti - sN;                                                                                %Terminos de intercambio
pi    = piD + alfa*(ti-ti(-1));                                                                 %CPI inflacin
y     = a + (1-alfaM)*n;                                                                        %Funcin de produccin (empleo)
rR    = i - piD(+1);                                                                            %Tasa de inters real
w-p   = sigma*c+fi*n;                                                                           %FOC salario
nx    = alfa*(omega/sigma-1)*ti - alfa/sigma*g;                                                 %Exportaciones netas
nx    = y - c - alfa*ti;                                                                        %Consumo determinado por la restriccin de recursos
yS    = 0;                                                                                      %Choque de crecimiento del producto mundial
yH    = y - steady_state(y);                                                                    %Desviacin de producto desde su estado estacionario
pD    = pD(-1) + piD;                                                                           %Nivel de precios domstico
p     = p(-1) + pi;                                                                             %IPC
ti     = er + pS -  pD;                                                                         %Tipo de cambio nominal
Der   = er - er(-1);                                                                            %Desviacin del tipo de cambio nominal
M    = rhoM*M(-1) + eM;                                                                         %Choqhe de politica monetaria
a     = rhoA*a(-1) + eA;                                                                        %Choque tecnolgico
g     = rhoG*g(-1) + eG;                                                                        %Choque de preferencias
iS    = rhoIS*iS(-1) + eIS;                                                                     %Choque de tasa de inters internacional
pS    = rhoPS*pS(-1) + ePS;                                                                     %Choque de tasa de inters internacional

@#if REGLA6 == 1                                                                                  
    [name='Interest Rate Rule (REGLA6)']
    i = phiR*i(-1) + phiP*piD + phiX*x + M;
@#else
    @#if REGLA7 ==1
        [name='REGLA7']
        i = phiR*i(-1) + phiPF*piD(+1) + M; 
    @#else
        @#if REGLA8 ==1
            [name='REGLA8']
            i = phiR*i(-1) + phiP*piD + phiER*er + M;
        @#else
            @#if REGLA9 ==1
                [name='REGLA9']
                i = phiR1*i(-1) + phiP0*piD + phiP1*piD(-1) + phiX0*x - phiX1*x(-1) + phiDER*Der + phiTI*ti + M;
            @#else
                @#if REGLA10 ==1
                    [name='REGLA10']
                    i = phiR*i(-1) + phiP*piD + phiX*x + phiER*er + M;  
                @#endif
            @#endif
        @#endif       
    @#endif
@#endif
end;

estimated_params;
   stderr eM, inv_gamma_pdf,0.1,0.02;
   stderr eA, inv_gamma_pdf,0.4,0.02;
   stderr eG, inv_gamma_pdf,0.2,0.02;
   stderr eIS, inv_gamma_pdf,0.2,0.02;
   stderr ePS, inv_gamma_pdf,1,0.02;
   rhoM, beta_pdf,0.5,0.01;
   rhoA, beta_pdf,0.9,0.01;
   rhoG, beta_pdf,0.9,0.01;
   rhoIS, beta_pdf,0.9,0.01;
   rhoPS, beta_pdf,0.95,0.01;
end;



%----------------------------------------------------------------
% Variables observadas en la estimacin
%----------------------------------------------------------------
varobs y c pi i;

%----------------------------------------------------------------
% Estimacin
%----------------------------------------------------------------

estimation(datafile=dsgeFF,mode_check,mode_compute=6,mh_replic=250000,mh_nblocks=2,bayesian_irf,lik_init=2,nograph) x y piD pi pD p ti er il i rR M a g iS pS;
oo_regla8_log = oo_; 
save('oo_regla8_log.mat', 'oo_regla8_log');
load oo_regla8_log.mat; 
compareMHMCMCLaplace;