%Reglas de poltica monetaria para una economa abierta
%Autor: Augusto Aliaga
%Fecha: Mayo de 2020
@#define REGLA1 =0
@#define REGLA2 =0
@#define REGLA3 =0
@#define REGLA4 =0
@#define REGLA5 =1

% Cadenas de cada regla de poltica monetaria
@#if REGLA1 == 1
    case_title='REGLA1'; 
    short_title='R1';   
@#else
    @#if REGLA2 ==1
         case_title='REGLA2';
         short_title='R2';
    @#else
        @#if REGLA3 ==1
            case_title='REGLA3';
            short_title='R3';   
        @#else
            @#if REGLA4 ==1
                case_title='REGLA4';
                short_title='R4';
            @#else
                @#if REGLA5 ==1
                     case_title='REGLA5';
                     short_title='R5';
                @#else
                    error
                @#endif
            @#endif
        @#endif       
    @#endif
@#endif
var x                                                               %Brecha del producto
    piD                                                             %Inflacin domstica
    i                                                               %Tasa de inters nominal
    il                                                              %Tasa de interes a largo plazo
    bl                                                              %Bonos a largo plazo
    yN                                                              %Producto natural
    rN                                                              %Tasa de inters natural
    sN                                                              %Terminos de intercambio natural
    sB                                                              %Brecha de trminos de intercambio
    y                                                               %Producto
    ti                                                              %Trminos de intercambio
    pi                                                              %CPI inflacin
    n                                                               %Empleo
    rR                                                              %Tasa de inters real   
    w                                                               %Salario nominal  
    nx                                                              %Exportaciones netas
    c                                                               %Consumo   
    yS                                                              %Producto mundial
    yH                                                              %Desviacin de producto desde su estado estacionario
    pD                                                              %ndice de precios domstico
    p                                                               %CPI
    er                                                              %Tipo de cambio nominal
    Der                                                             %Desviacin del tipo de cambio nominal
    M                                                               %Proceso AR(1) del choque de politica monetaria
    a                                                               %Proceso AR(1) del choque tecnolgico
    g                                                               %Proceso AR(1) del choque de preferencias
    iS                                                              %Proceso AR(1) del choque de tasa de inters internacional
    pS                                                              %Nivel de precios mundial
    ;

varexo eM                                                           %Choque de politica monetaria
       eA                                                           %Choque tecnolgico
       eG                                                           %Choque de preferencias
       eIS                                                          %Choque de tasa de inters internacional
       ePS                                                          %Choque del nivel de precios mundial
       ;

parameters beta                                                     %Factor de descuento
    sigma                                                           %inversa de la EIS
    fi                                                              %inversa de la elasticidad de Frisch
    alfaM                                                           %Proporcin del capital
    epsilon                                                         %Elasticidad de la demanda
    theta                                                           %Parmetro de Calvo 
    alfa                                                            %Parmetro de apertura
    eta                                                             %substitutability foreign/domestic goods
    rhoA                                                            %Parmetro del choque tecnolgico
    rhoM                                                            %Parmetro del choque de poltica monetaria
    rhoG                                                            %Parmetro del choque de preferencias
    rhoIS                                                           %Parmetro del choque de tasa de inters internacional
    rhoPS                                                           %Parmetro del choque del nivel de precios internacional
    phiP                                                            %Reaccin a la inflacin
    phiX                                                            %Reaccin a la brecha del producto
    phiR                                                            %Reaccin al rezago de la tasa de inters nominal
    phiPF                                                           %Reaccin a la inflacin domstica proyectada
    phiER                                                           %Reaccin al tipo de cambio en la regla monetaria
    delta                                                           %Tasa de los costos de ajuste
    ;

%----------------------------------------------------------------
% Parametrizacin
%----------------------------------------------------------------
beta    = 0.9852;
sigma   = 1;
fi      = 0.45;
alfaM   = 0;
epsilon = 6;
theta   = 0.75;
alfa    = 0.6;
rhoM  = 0.5;
rhoA   = 0.9;
rhoG   = 0.9;
rhoIS  = 0.9;
rhoPS  = 0.95;
phiP    = 1.5;
phiX    = 0.5;
phiR    = 0.5;
phiPF   = 5;
phiER   = 0.5;
eta     = 2;
delta   = 3;

%----------------------------------------------------------------
% Modelo log-lineal
%----------------------------------------------------------------
model(linear); 
%Parmetros compuestos
#omicron   = (1-alfaM)/(1-alfaM+alfaM*epsilon);           
#lambda    = (1-theta)*(1-beta*theta)/theta*omicron;     
#omega     = sigma*eta + (1-alfa)*(sigma*eta-1);         
#fim       = 1/(1+alfa*(omega-1));                    
#sigmaA    = sigma*fim;                                  
#gammaA    = (1+fi)/(sigmaA*(1-alfaM)+fi+alfaM);                                            
#gammaS    = - alfa*(omega-1)*sigmaA*(1-alfaM)/(sigmaA*(1-alfaM)+fi+alfaM);           
#gammaG    = - alfa*omega*fim*(1 - alfaM)/(sigmaA*(1-alfaM)+fi+alfaM);                       
#kappaA    = lambda*(sigmaA+(fi+alfaM)/(1 - alfaM));                                           
#psiS      = sigmaA*(alfa*(omega-1)+gammaS);                                                
#psiG      = (1-alfa)*fim - sigmaA*gammaG;
#gamma1   =   (1+delta+beta);                                       
#gamma2    = ((1+delta)^2)/delta;                                                                             
x     = x(+1) - 1/sigmaA*((1/(1+delta))*i - piD(+1) + (delta/(1+delta))*il(+1) - rN);           %Curva IS dinmica
piD   = beta*piD(+1) + kappaA*x;                                                                %Curva de Phillips Mevo Keynesiana                                                                                    
il    = i(-1) + gamma1*gamma2*bl(-1) - gamma2*bl(-2) - beta*gamma2*bl;                          %Tasa de interes a largo plazo
i     = iS + er(+1) - er;                                                                       %Paridad descubierta de tasas de interes
yN    = gammaA*a + gammaG*g + gammaS*yS;                                                        %Producto natural
rN    = -sigmaA*gammaA*(1-rhoA)*a + psiS*(yS(+1)-yS) + psiG*(1-rhoG)*g;                         %Tasa de inters natural
sN    = sigmaA*(yN-yS) - (1-alfa)*fim*g;                                                        %Terminos de intercambio natural                                             
sB    = sigmaA*x;                                                                               %Brecha de trminos de intercambio
x     = y - yN;                                                                                 %Producto
sB    = ti - sN;                                                                                %Terminos de intercambio
pi    = piD + alfa*(ti-ti(-1));                                                                 %CPI inflacin
y     = a + (1-alfaM)*n;                                                                        %Funcin de produccin (empleo)
rR    = i - piD(+1);                                                                            %Tasa de inters real
w-p   = sigma*c+fi*n;                                                                           %FOC salario
nx    = alfa*(omega/sigma-1)*ti - alfa/sigma*g;                                                 %Exportaciones netas
nx    = y - c - alfa*ti;                                                                        %Consumo determinado por la restriccin de recursos
yS    = 0;                                                                                      %Choque de crecimiento del producto mundial
yH    = y - steady_state(y);                                                                    %Desviacin de producto desde su estado estacionario
pD    = pD(-1) + piD;                                                                           %Nivel de precios domstico
p     = p(-1) + pi;                                                                             %IPC
ti     = er + pS -  pD;                                                                         %Tipo de cambio nominal
Der   = er - er(-1);                                                                            %Desviacin del tipo de cambio nominal
M    = rhoM*M(-1) + eM;                                                                         %Choqhe de politica monetaria
a     = rhoA*a(-1) + eA;                                                                        %Choque tecnolgico
g     = rhoG*g(-1) + eG;                                                                        %Choque de preferencias
iS    = rhoIS*iS(-1) + eIS;                                                                     %Choque de tasa de inters internacional
pS    = rhoPS*pS(-1) + ePS;                                                                     %Choque de tasa de inters internacional

%Reglas de poltica monetaria           
@#if REGLA1 == 1                                                                                  
    [name='Interest Rate Rule (REGLA1)']
    i=phiP*pi + M;
@#else
    @#if REGLA2 ==1
        [name='REGLA2']
        i = phiP*piD + M;
    @#else
        @#if REGLA3 ==1
            [name='REGLA3']
            er = 0;
        @#else
            @#if REGLA4 ==1
                [name='REGLA4']
                i = phiP*pi + phiX*x + M; 
            @#else
                @#if REGLA5 ==1
                    [name='REGLA5']
                    i = phiP*piD + phiX*x + M;  
                @#endif
            @#endif
        @#endif       
    @#endif
@#endif
end;

shocks;
    var eM = 1;       %Choque del 1 por ciento
    var eA = 1;
    var eG = 1;
    var eIS = 1; 
    var ePS = 1;
end;

%----------------------------------------------------------------
% Generar las FIRs
%----------------------------------------------------------------
stoch_simul(order = 1,irf=40,graph_format=pdf)  x y bl piD pi pD p ti er Der i il rR M a g iS pS;

%----------------------------------------------------------------
% Generar el Cuadro 4.2.
%----------------------------------------------------------------

%Varianzas del resultado
y_pos=strmatch('y',var_list_ ,'exact');
x_pos=strmatch('x',var_list_ ,'exact');
pi_pos=strmatch('pi',var_list_ ,'exact');
piD_pos=strmatch('piD',var_list_ ,'exact');
ti_pos=strmatch('ti',var_list_ ,'exact');
Der_pos=strmatch('Der',var_list_ ,'exact');

%Conseguir parmetros
par.alfa=M_.params(strmatch('alfa',M_.param_names,'exact'));
par.theta=M_.params(strmatch('theta',M_.param_names,'exact'));
par.alfaM=M_.params(strmatch('alfaM',M_.param_names,'exact'));
par.beta=M_.params(strmatch('beta',M_.param_names,'exact'));
par.epsilon=M_.params(strmatch('epsilon',M_.param_names,'exact'));
par.sigma=M_.params(strmatch('sigma',M_.param_names,'exact'));
par.fi=M_.params(strmatch('fi',M_.param_names,'exact'));
par.lambda=(1-par.theta)*(1-par.beta*par.theta)/par.theta*(1-par.alfaM)/(1-par.alfaM+par.alfaM*par.epsilon);

%Leer varianzas
variance.y=oo_.var(y_pos,y_pos);
variance.x=oo_.var(x_pos,x_pos);
variance.piD=oo_.var(piD_pos,piD_pos);
variance.pi=oo_.var(pi_pos,pi_pos);
variance.ti=oo_.var(ti_pos,ti_pos);
variance.Der=oo_.var(Der_pos,Der_pos);

%Funcin de prdida
L=(1-par.alfa)*0.5*(((1+par.fi)/(1-par.alfaM))*variance.x+par.epsilon/par.lambda*variance.piD)/100;

%Resultados del Cuadro 4.2.
labels=strvcat('sigma(y)','sigma(x)','sigma(piD)','sigma(pi)','sigma(ti)','sigma(Delta e)','L');
headers=strvcat(' ',short_title);
values=[sqrt(variance.y);sqrt(variance.x);sqrt(variance.piD);sqrt(variance.pi);sqrt(variance.ti);sqrt(variance.Der);L];
options_.noprint=0;
dyntable(options_,'Cuadro 4.2: Comparacin entre reglas de poltica monetaria',headers,labels,values,size(labels,2)+2,4,3)